==> python-language-24.2.0-sources.jar: com/oracle/graal/python/nodes/argument/ReadVarArgsNode.java <==
/*
 * Copyright (c) 2017, 2021, Oracle and/or its affiliates.
 * Copyright (c) 2013, Regents of the University of California
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are
 * permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice, this list of
 * conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other materials provided
 * with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package com.oracle.graal.python.nodes.argument;

import com.oracle.graal.python.builtins.objects.function.PArguments;
import com.oracle.graal.python.runtime.object.PythonObjectFactory;
import com.oracle.truffle.api.dsl.Specialization;
import com.oracle.truffle.api.frame.VirtualFrame;

public abstract class ReadVarArgsNode extends ReadArgumentNode {
    @Child private PythonObjectFactory factory;

    ReadVarArgsNode(boolean isBuiltin) {
        factory = isBuiltin ? null : PythonObjectFactory.create();
    }

    public static ReadVarArgsNode create() {
        return create(false);
    }

    public static ReadVarArgsNode create(boolean isBuiltin) {
        return ReadVarArgsNodeGen.create(isBuiltin);
    }

    public abstract Object[] executeObjectArray(VirtualFrame frame);

    @Specialization
    Object extractVariableVarargs(VirtualFrame frame) {
        return output(PArguments.getVariableArguments(frame));
    }

    private Object output(Object[] varArgs) {
        if (isBuiltin()) {
            return varArgs;
        } else {
            return factory.createTuple(varArgs);
        }
    }

    public boolean isBuiltin() {
        return factory == null;
    }
}

==> python-language-25.0.2-sources.jar: com/oracle/graal/python/nodes/argument/ReadVarArgsNode.java <==
/*
 * Copyright (c) 2017, 2025, Oracle and/or its affiliates.
 * Copyright (c) 2013, Regents of the University of California
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are
 * permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice, this list of
 * conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other materials provided
 * with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package com.oracle.graal.python.nodes.argument;

import com.oracle.graal.python.PythonLanguage;
import com.oracle.graal.python.builtins.objects.function.PArguments;
import com.oracle.graal.python.runtime.object.PFactory;
import com.oracle.truffle.api.dsl.Specialization;
import com.oracle.truffle.api.frame.VirtualFrame;

public abstract class ReadVarArgsNode extends ReadArgumentNode {
    private final boolean isBuiltin;

    ReadVarArgsNode(boolean isBuiltin) {
        this.isBuiltin = isBuiltin;
    }

    public static ReadVarArgsNode create() {
        return create(false);
    }

    public static ReadVarArgsNode create(boolean isBuiltin) {
        return ReadVarArgsNodeGen.create(isBuiltin);
    }

    public abstract Object[] executeObjectArray(VirtualFrame frame);

    @Specialization
    Object extractVariableVarargs(VirtualFrame frame) {
        return output(PArguments.getVariableArguments(frame));
    }

    private Object output(Object[] varArgs) {
        if (isBuiltin()) {
            return varArgs;
        } else {
            return PFactory.createTuple(PythonLanguage.get(this), varArgs);
        }
    }

    public boolean isBuiltin() {
        return isBuiltin;
    }
}
